@@ -1,20 +1,29 @@
-# $Id: BaseObject.pm 357 2007-05-01 04:27:29Z miyagawa $
+# $Id$
 
 package Data::ObjectDriver::BaseObject;
 use strict;
 use warnings;
 
-use Scalar::Util qw(weaken);
+our $HasWeaken;
+eval q{ use Scalar::Util qw(weaken) }; ## no critic
+$HasWeaken = !$@;
+
 use Carp ();
 
 use Class::Trigger qw( pre_save post_save post_load pre_search
                        pre_insert post_insert pre_update post_update
                        pre_remove post_remove post_inflate );
 
+use Data::ObjectDriver::ResultSet;
+
+## Global Transaction variables
+our @WorkingDrivers;
+our $TransactionLevel = 0;
+
 sub install_properties {
     my $class = shift;
     my($props) = @_;
-    my $columns = delete $props->{columns}; 
+    my $columns = delete $props->{columns};
     $props->{columns} = [];
     {
         no strict 'refs'; ## no critic
@@ -110,8 +119,11 @@ sub has_a {
                     ;
                 ## Hold in a variable here too, so we don't lose it immediately
                 ## by having only the weak reference.
-                my $ret = $obj->{$cachekey} = $parentclass->lookup($id);
-                weaken $obj->{$cachekey};
+                my $ret = $parentclass->lookup($id);
+                if ($HasWeaken) {
+                    $obj->{$cachekey} = $ret;
+                    weaken($obj->{$cachekey});
+                }
                 return $ret;
             };
         } else {
@@ -261,6 +273,27 @@ sub primary_key_to_terms {
     \%terms;
 }
 
+sub is_same {
+    my($obj, $other) = @_;
+
+    my @a;
+    for my $o ($obj, $other) {
+        push @a, [ map { $o->$_() } @{ $o->primary_key_tuple }];
+    }
+    return is_same_array( @a );
+}
+
+sub object_is_stored {
+    my $obj = shift;
+    return $obj->{__is_stored} ? 1 : 0;
+}
+sub pk_str {
+    my ($obj) = @_;
+    my $pk = $obj->primary_key;
+    return $pk unless ref ($pk) eq 'ARRAY';
+    return join (":", @$pk);
+}
+
 sub has_primary_key {
     my $obj = shift;
     return unless @{$obj->primary_key_tuple};
@@ -292,7 +325,7 @@ sub set_values {
     my $values = shift;
     for my $col (keys %$values) {
         unless ( $obj->has_column($col) ) {
-            Carp::croak("You tried to set inexistent column $col to value $values->{$col} on " . ref($obj));
+            Carp::croak("You tried to set non-existent column $col to value $values->{$col} on " . ref($obj));
         }
         $obj->$col($values->{$col});
     }
@@ -443,28 +476,60 @@ sub lookup_multi {
     $objs;
 }
 
+sub result {
+    my $class = shift;
+    my ($terms, $args) = @_;
+
+    return Data::ObjectDriver::ResultSet->new({
+                          class     => (ref $class || $class),
+                          page_size => delete $args->{page_size},
+                          paging    => delete $args->{no_paging},
+                          terms     => $terms,
+                          args      => $args,
+                          });
+}
+
 sub search {
     my $class = shift;
     my($terms, $args) = @_;
     my $driver = $class->driver;
-    my @objs = $driver->search($class, $terms, $args);
+    if (wantarray) {
+        my @objs = $driver->search($class, $terms, $args);
 
-    ## Don't attempt to cache objects where the caller specified fetchonly,
-    ## because they won't be complete.
-    ## Also skip this step if we don't get any objects back from the search
-    if (!$args->{fetchonly} || !@objs) {
-        for my $obj (@objs) {
-            $driver->cache_object($obj) if $obj;
+        ## Don't attempt to cache objects where the caller specified fetchonly,
+        ## because they won't be complete.
+        ## Also skip this step if we don't get any objects back from the search
+        if (!$args->{fetchonly} || !@objs) {
+            for my $obj (@objs) {
+                $driver->cache_object($obj) if $obj;
+            }
         }
+        return @objs;
+    } else {
+        my $iter = $driver->search($class, $terms, $args);
+        return $iter if $args->{fetchonly};
+
+        my $caching_iter = sub {
+            my $d = $driver;
+
+            my $o = $iter->();
+            unless ($o) {
+                $iter->end;
+                return;
+            }
+            $driver->cache_object($o);
+            return $o;
+        };
+        return Data::ObjectDriver::Iterator->new($caching_iter, sub { $iter->end });
     }
-    $driver->list_or_iterator(\@objs);
 }
 
-sub remove          { shift->_proxy('remove',       @_) }
-sub update          { shift->_proxy('update',       @_) }
-sub insert          { shift->_proxy('insert',       @_) }
-sub replace         { shift->_proxy('replace',      @_) }
-sub fetch_data      { shift->_proxy('fetch_data',   @_) }
+sub remove         { shift->_proxy( 'remove',         @_ ) }
+sub update         { shift->_proxy( 'update',         @_ ) }
+sub insert         { shift->_proxy( 'insert',         @_ ) }
+sub replace        { shift->_proxy( 'replace',        @_ ) }
+sub fetch_data     { shift->_proxy( 'fetch_data',     @_ ) }
+sub uncache_object { shift->_proxy( 'uncache_object', @_ ) }
 
 sub refresh {
     my $obj = shift;
@@ -472,13 +537,78 @@ sub refresh {
     my $fields = $obj->fetch_data;
     $obj->set_values_internal($fields);
     $obj->call_trigger('post_load');
+    $obj->driver->cache_object($obj);
     return 1;
 }
 
+## NOTE: I wonder if it could be useful to BaseObject superclass
+## to override the global transaction flag. If so, I'd add methods
+## to manipulate this flag and the working drivers. -- Yann
 sub _proxy {
     my $obj = shift;
     my($meth, @args) = @_;
-    $obj->driver->$meth($obj, @args);
+    my $driver = $obj->driver;
+    ## faster than $obj->txn_active && ! $driver->txn_active but see note.
+    if ($TransactionLevel && ! $driver->txn_active) {
+        $driver->begin_work;
+        push @WorkingDrivers, $driver;
+    }
+    $driver->$meth($obj, @args);
+}
+
+sub txn_active { $TransactionLevel }
+
+sub begin_work {
+    my $class = shift;
+    if ( $TransactionLevel > 0 ) {
+        Carp::carp(
+            $TransactionLevel > 1
+            ? "$TransactionLevel transactions already active"
+            : "Transaction already active"
+        );
+    }
+    $TransactionLevel++;
+}
+
+sub commit {
+    my $class = shift;
+    $class->_end_txn('commit');
+}
+
+sub rollback {
+    my $class = shift;
+    $class->_end_txn('rollback');
+}
+
+sub _end_txn {
+    my $class = shift;
+    my $meth  =  shift;
+    
+    ## Ignore nested transactions
+    if ($TransactionLevel > 1) {
+        $TransactionLevel--;
+        return;
+    }
+    
+    if (! $TransactionLevel) {
+        Carp::carp("No active transaction to end; ignoring $meth");
+        return;
+    }
+    my @wd = @WorkingDrivers;
+    $TransactionLevel--;
+    @WorkingDrivers = ();
+    
+    for my $driver (@wd) {
+        $driver->$meth;
+    }
+}
+
+sub txn_debug {
+    my $class = shift;
+    return {
+        txn     => $TransactionLevel,
+        drivers => \@WorkingDrivers,
+    };
 }
 
 sub deflate { { columns => shift->column_values } }
@@ -487,8 +617,7 @@ sub inflate {
     my $class = shift;
     my($deflated) = @_;
     my $obj = $class->new;
-    $obj->set_values($deflated->{columns});
-    $obj->{changed_cols} = {};
+    $obj->set_values_internal($deflated->{columns});
     $obj->call_trigger('post_inflate');
     return $obj;
 }
@@ -635,7 +764,8 @@ Known C<column_defs> types are:
 =item * C<blob>
 
 A blob of binary data. C<Data::ObjectDriver::Driver::DBD::Pg> maps this to
-C<DBI::Pg::PG_BYTEA>, and C<DBD::SQLite> to C<DBI::SQL_BLOB>.
+C<DBI::Pg::PG_BYTEA>, C<DBD::SQLite> to C<DBI::SQL_BLOB> and C<DBD::Oracle>
+to C<ORA_BLOB>.
 
 =item * C<bin_char>
 
@@ -822,6 +952,12 @@ C<SELECT> query will include a C<FOR UPDATE> clause.
 All options are passed to the object driver, so your driver may support
 additional options.
 
+=head2 C<Class-E<gt>result(\%terms, [\%args])>
+
+Takes the same I<%terms> and I<%args> arguments that I<search> takes, but
+instead of executing the query immediately, returns a
+I<Data::ObjectDriver::ResultSet> object representing the set of results.
+
 =head2 C<$obj-E<gt>exists()>
 
 Returns true if C<$obj> already exists in the database.
@@ -875,7 +1011,7 @@ accessors than reading the properties directly.
 =head2 C<Class-E<gt>driver()>
 
 Returns the object driver for this class, invoking the class's I<get_driver>
-function (and caching the result for future calls) if necessary. 
+function (and caching the result for future calls) if necessary.
 
 =head2 C<Class-E<gt>get_driver($get_driver_fn)>
 
@@ -903,10 +1039,31 @@ Returns the I<values> of the primary key fields of C<$obj>.
 
 Returns the I<names> of the primary key fields of C<Class> objects.
 
+=head2 C<$obj-E<gt>is_same($other_obj)>
+
+Do a primary key check on C<$obj> and $<other_obj> and returns true only if they
+are identical.
+
+=head2 C<$obj-E<gt>object_is_stored()>
+
+Returns true if the object hasn't been stored in the database yet.
+This is particularily useful in triggers where you can then determine
+if the object is being INSERTED or just UPDATED.
+
+=head2 C<$obj-E<gt>pk_str()>
+
+returns the primay key has a printable string.
+
 =head2 C<$obj-E<gt>has_primary_key()>
 
 Returns whether the given object has values for all of its primary key fields.
 
+=head2 C<$obj-E<gt>uncache_object()>
+
+If you use a Cache driver, returned object will be automatically cached as a result
+of common retrieve operations. In some rare cases you may want the cache to be cleared
+explicitely, and this method provides you with a way to do it.
+
 =head2 C<$obj-E<gt>primary_key_to_terms([$id])>
 
 Returns C<$obj>'s primary key as a hashref of values keyed on column names,
@@ -1026,6 +1183,41 @@ Inflates the deflated representation of the object I<$deflated> into a proper
 object in the class I<Class>. That is, undoes the operation C<$deflated =
 $obj-E<gt>deflate()> by returning a new object equivalent to C<$obj>.
 
+=head1 TRANSACTION SUPPORT AND METHODS
+
+=head2 Introduction
+
+When dealing with the methods on this class, the transactions are global,
+i.e: applied to all drivers. You can still enable transactions per driver
+if you directly use the driver API.
+
+=head2 C<Class-E<gt>begin_work>
+
+This enable transactions globally for all drivers until the next L<rollback>
+or L<commit> call on the class.
+
+If begin_work is called while a transaction is still active (nested transaction)
+then the two transactions are merged. So inner transactions are ignored and
+a warning will be emitted.
+
+=head2 C<Class-E<gt>rollback>
+
+This rollbacks all the transactions since the last begin work, and exits
+from the active transaction state.
+
+=head2 C<Class-E<gt>commit>
+
+Commits the transactions, and exits from the active transaction state.
+
+=head2 C<Class-E<gt>txn_debug>
+
+Just return the value of the global flag and the current working drivers
+in a hashref.
+
+=head2 C<Class-E<gt>txn_active>
+
+Returns true if a transaction is already active.
+
 =head1 DIAGNOSTICS
 
 =over 4
