@@ -90,17 +90,20 @@ sub init_db {
 sub rw_handle {
     my $driver = shift;
     my $db = shift || 'main';
-    $driver->dbh(undef) if $driver->dbh and !$driver->dbh->ping;
-    my $dbh = $driver->dbh;
-    unless ($dbh) {
-        if (my $getter = $driver->get_dbh) {
-            $dbh = $getter->();
-        } else {
-            $dbh = $driver->init_db($db) or die $driver->last_error;
-            $driver->dbh($dbh);
-        }
+    my $dbh;
+    if ($dbh = $driver->dbh) {
+        return $dbh if $dbh->ping;
+
+        ## ping fails, kill cache.
+        delete $Handles{$driver->dsn};
     }
-    $dbh;
+    if (my $getter = $driver->get_dbh) {
+        $dbh = $getter->();
+    } else {
+        $dbh = $driver->init_db($db) or die $driver->last_error;
+        $driver->dbh($dbh);
+    }
+    return $dbh;
 }
 *r_handle = \&rw_handle;
 
